# FILE: savedlg.tcl
#
# Save file dialog box for mmDisp
#
# Callbacks: SAVE
#            DELETE $delete_callback_arg
#
# debugging enable/disable/level

Oc_Class SaveDlg {
    const public variable winpath
    const public variable outer_frame_options = "-bd 2 -relief raised"
    const public variable filter = "*.o?f *.svf *.vio"
    const public variable dialog_title = "Save File"
    const public variable mesh_update_proc = Oc_Nop
    const public variable mesh_update_variable = {} ;# If non-nil, then
    ## a w trace gets setup on this global (name) to automatically
    ## update "Regular" gridtype availability, and also file title
    ## and description fields.  New values are obtained by calling
    ## [eval $mesh_update_proc cget -type], ...-title, and 
    ## ...-description.  The returned gridtype should be one of
    ## "Vf_GridVec3f", "Vf_GeneralMesh3f", or "Vf_Empty".
    public variable callback = Oc_Nop
    public variable delete_callback_arg = {}
    private variable filewidget = {}
    private variable optsbox = {}
    private variable ctrlwidget = {}
    private variable descbox = {}
    const private variable titlelabel = "Title:"
    const private variable desclabel = "Desc:"
    private variable selectionwidget = {}
    private variable dirwidget = {}

    # The following variables control filename checks performed on
    # SELECT action before calling $callback.
    public variable dir_must_exist = 1
    public variable file_must_exist = 0
    public variable valid_file_types = {file fifo socket}
    ## Set valid_file_types to {} to disable file type checking.
    ## Refer to the Tcl 'file type' command for a list of possible
    ## file types.  NOTE: If 'file type' returns "link", and "link"
    ## is not on this list, then Oc_ResolveLink is called to
    ## the underlying file type, which is then recompared to this
    ## list.  If you don't *ever* want links, then put "link" on
    ## this list and check the file type explicitly in the callback.


    Constructor {args} {
	eval $this Configure -winpath .w$this $args
	# Create toplevel frame to hold dialog
	toplevel $winpath
	wm group $winpath .
	wm title $winpath $dialog_title
	if {![string match {} [info commands Ow_PositionChild]]} {
	    Ow_PositionChild $winpath  ;# Position at +.25+.25 over '.'
	}
	wm protocol $winpath WM_DELETE_WINDOW \
		"$this SwitchBoard $this CLOSE"
      focus $winpath

	# Temporarily disable callbacks until construction is complete
	set callback_keep $callback ;	set callback Oc_Nop

	# Create subwidgets
	Ow_FileSelect New filewidget $winpath \
		-callback "$this SwitchBoard" \
		-filter $filter
	set ctrlframe [frame $winpath.ctrlframe]
	OmfOpts New optsbox $winpath
	CtrlWBrowse New ctrlwidget $winpath \
		-callback "$this SwitchBoard"
	Ow_MultiBox New descbox $winpath -labelwidth 5 \
		-valuewidth 10 \
		-labellist [list [list $titlelabel {}] [list $desclabel {}]]
	Ow_SelectBox New selectionwidget $winpath \
		-title "Save File As..." \
		-callback "$this SwitchBoard"
	Ow_DirSelect New dirwidget $winpath \
		-callback "$this SwitchBoard"

	# Initialize selection path
	$selectionwidget SetPath [$dirwidget Cget -value]

	# Pack subwidgets
	pack [$filewidget Cget -winpath] -side left \
		-fill both -expand 1 -padx 2m -pady 2m
	pack [$ctrlwidget Cget -winpath] -in $ctrlframe \
		-side right -expand 0 -anchor ne -padx 2m -pady 2m
	pack [$optsbox Cget -winpath] -in $ctrlframe \
		-side top -expand 0 -padx 2m -pady 2m
	pack $ctrlframe -side top \
		-fill x -expand 0 -anchor ne -padx 2m -pady 2m
	pack [$descbox Cget -winpath] -side top \
		-fill x -expand 0 -anchor ne -padx 2m -pady 2m
	pack [$selectionwidget Cget -winpath] -side top \
		-fill x -expand 0 -anchor n -padx 2m -pady 2m
	pack [$dirwidget Cget -winpath] -side right \
		-fill both -expand 1 -padx 2m -pady 2m

	# Widget bindings
	bind $winpath <Destroy> "+$this WinDestroy %W"

	# Mesh (grid) types setup and trace
	if { ![ string match {} mesh_update_variable ] } {
	    upvar #0 $mesh_update_variable meshchange
	    if {![info exists meshchange]} { set meshchange {} }
	    $this MeshChangeTrace $mesh_update_variable {} w
	    trace variable meshchange w "$this MeshChangeTrace"
	}

	# Re-enable callbacks
	set callback $callback_keep
    }
    method MeshChangeTrace { name elt op } { \
	    mesh_update_variable mesh_update_proc \
	    optsbox descbox titlelabel desclabel } {
	set gridtype [eval $mesh_update_proc cget -gridtype]
	if { [string match "Vf_GridVec3f" $gridtype] } {
	    $optsbox GridsAllowedUpdate 1
	} else {
	    $optsbox GridsAllowedUpdate 0
	}
	$descbox Set $titlelabel [eval $mesh_update_proc cget -title]
	$descbox Set $desclabel [eval $mesh_update_proc cget -description]
    }
    method GetOptionState {} { optsbox } {
	# Returns a list of option box state pairs,
	# e.g., -datatype binary4 -gridtype regular
	return [$optsbox GetState]
    }
    method GetDescStrings {} { descbox titlelabel desclabel } {
	# Returns a list of description string pairs,
	# e.g., -title "This is the title" -desc "Any description"
	set desclist {}
	lappend desclist -title [$descbox LookUp $titlelabel]
	lappend desclist -desc [$descbox LookUp $desclabel]
	return $desclist
    }
    method GetFilename {} { selectionwidget } {
	# Returns full path+filename of selected value
	return [$selectionwidget ReadDisplayedFullname]
    }
    method SwitchBoard { subwidget actionid args } {
	# General callback routing routine
	switch -exact -- $actionid {
	    UPDATE {
		# Update actions
		if {[string compare $subwidget $filewidget]==0} {
			$selectionwidget SetFilename [lindex $args 1]
		} elseif {[string compare $subwidget $dirwidget]==0} {
			$filewidget SetPath [lindex $args 1]
			$selectionwidget SetPath [lindex $args 1]
		} else {
		    error "Unknown UPDATE callback source: [\
			    $subwidget class]"
		}
	    }
	    SELECT {
		# Check that selected filename is valid
		set fname [$this GetFilename]
		if {$dir_must_exist \
			&& ![file isdirectory [file dirname $fname]]} {
		    error "Filename error: [file dirname $fname]\
			    is not a valid directory"
		}
		if {[file exists $fname]} {
		    # File exists.  Check that it is of valid type
		    if {![string match {} $valid_file_types]} {
			set ftype [file type $fname]
			if {[lsearch -exact $valid_file_types $ftype]<0} {
			    set badtype 1
			    if {[string match "link" $fname]} {
				# Resolve link and compare that type
				if {![catch {Oc_ResolveLink $fname} ttype] \
					&& [lsearch -exact \
					    $valid_file_types $ttype]<0} {
				    set ftype $ttype
				    set badtype 0
				}
			    }
			    if {$badtype} {
				error "File $fname has invalid type:\
					$ftype\nShould be one of:\
					$valid_file_types"
			    }

			}
		    }
		} elseif {$file_must_exist} {
		    error "File $fname does not exist"
		}

		set errmsg [eval $callback $this SAVE]
		if { ![$ctrlwidget Cget -browse_state] \
			&& [string match {} $errmsg] } {
		    $this Delete ; return
		}
		$filewidget Rescan
		$dirwidget Rescan
	    }
	    RESCAN {
		$filewidget Rescan
		$dirwidget Rescan
	    }
	    CLOSE {
		$this Delete ; return
	    }
	    DELETE {}
	    default { error "Unknown actionid: $actionid" }
	}
    }
    method WinDestroy { w } { winpath } {
	if {[info exists winpath] && [string compare $winpath $w]!=0} {
	    return    ;# Child destroy event
	}
	$this Delete
    }
    private variable delete_in_progress = 0
    Destructor {
	# Protect against re-entrancy
	if { [info exists delete_in_progress] && \
		$delete_in_progress } { return }
	set delete_in_progress 1

	# Notify client of impending destruction
	catch {eval $callback $this DELETE {$delete_callback_arg}}

	# Remove meshtype trace
	catch {
	    if { ![ string match {} mesh_update_variable ] } {
		upvar #0 $mesh_update_variable meshchange
		trace vdelete meshchange w "$this MeshChangeTrace"
	    }
	}

	# Delete subwidgets
	catch {$filewidget      Delete}
	catch {$ctrlwidget      Delete}
	catch {$selectionwidget Delete}
	catch {$dirwidget       Delete}

	# Delete window
	if {[info exists winpath] && [winfo exists $winpath]} {
	    # Destroy instance frame, all children, and bindings
	    destroy $winpath
	}
    }
}
